@@ -20,7 +20,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 #ifndef __JackChannel__
 #define __JackChannel__
 
-#include "session.h"
+#include "types.h"
+#include "JackSession.h"
+#include "JackMetadata.h"
 
 namespace Jack
 {
@@ -34,6 +36,54 @@ class JackGraphManager;
 namespace detail
 {
 
+class JackChannelTransactionInterface
+{
+
+    public:
+
+        JackChannelTransactionInterface()
+        {}
+        virtual ~JackChannelTransactionInterface()
+        {}
+
+        virtual int Read(void* data, int len) = 0;
+        virtual int Write(void* data, int len) = 0;
+
+};
+
+class JackRequestInterface
+{
+
+    public:
+
+        JackRequestInterface()
+        {}
+        virtual ~JackRequestInterface()
+        {}
+
+        virtual int Connect(const char* dir, const char* name, int which) = 0;
+        virtual int Close() = 0;
+
+};
+
+class JackClientRequestInterface : public JackChannelTransactionInterface, public JackRequestInterface
+{
+
+    public:
+
+        JackClientRequestInterface()
+        {}
+        virtual ~JackClientRequestInterface()
+        {}
+
+        virtual int Read(void* data, int len) { return -1; }
+        virtual int Write(void* data, int len) { return -1; }
+
+        virtual int Connect(const char* dir, const char* name, int which) { return -1; }
+        virtual int Close() { return -1; }
+
+};
+
 /*!
 \brief Inter process channel for server/client bidirectionnal communication : request and (receiving) notifications.
 */
@@ -49,7 +99,7 @@ class JackClientChannelInterface
         {}
 
         // Open the Server/Client connection
-        virtual int Open(const char* server_name, const char* name, int uuid, char* name_res, JackClient* obj, jack_options_t options, jack_status_t* status)
+        virtual int Open(const char* server_name, const char* name, jack_uuid_t uuid, char* name_res, JackClient* obj, jack_options_t options, jack_status_t* status)
         {
             return 0;
         }
@@ -73,9 +123,9 @@ class JackClientChannelInterface
             return -1;
         }
 
-        virtual void ClientCheck(const char* name, int uuid, char* name_res, int protocol, int options, int* status, int* result)
+        virtual void ClientCheck(const char* name, jack_uuid_t uuid, char* name_res, int protocol, int options, int* status, int* result, int open)
         {}
-        virtual void ClientOpen(const char* name, int pid, int uuid, int* shared_engine, int* shared_client, int* shared_graph, int* result)
+        virtual void ClientOpen(const char* name, int pid, jack_uuid_t uuid, int* shared_engine, int* shared_client, int* shared_graph, int* result)
         {}
         virtual void ClientOpen(const char* name, int* ref, JackEngineControl** shared_engine, JackGraphManager** shared_manager, JackClientInterface* client, int* result)
         {}
@@ -119,7 +169,7 @@ class JackClientChannelInterface
         {}
         virtual void InternalClientHandle(int refnum, const char* client_name, int* status, int* int_ref, int* result)
         {}
-        virtual void InternalClientLoad(int refnum, const char* client_name, const char* so_name, const char* objet_data, int options, int* status, int* int_ref, int uuid, int* result)
+        virtual void InternalClientLoad(int refnum, const char* client_name, const char* so_name, const char* objet_data, int options, int* status, int* int_ref, jack_uuid_t uuid, int* result)
         {}
         virtual void InternalClientUnload(int refnum, int int_ref, int* status, int* result)
         {}
@@ -137,6 +187,9 @@ class JackClientChannelInterface
         virtual void ClientHasSessionCallback(const char* client_name, int* result)
         {}
 
+        virtual void PropertyChangeNotify(jack_uuid_t subject, const char* key, jack_property_change_t change, int* result)
+        {}
+
         virtual bool IsChannelThread()
         {
             return false;
