@@ -25,7 +25,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 #include "JackPort.h"
 #include "JackSynchro.h"
 #include "JackNotification.h"
-#include "session.h"
+#include "JackSession.h"
 
 namespace Jack
 {
@@ -34,6 +34,7 @@ namespace Jack
 \brief Client control possibly in shared memory.
 */
 
+PRE_PACKED_STRUCTURE
 struct JackClientControl : public JackShmMemAble
 {
     char fName[JACK_CLIENT_NAME_SIZE + 1];
@@ -45,30 +46,31 @@ struct JackClientControl : public JackShmMemAble
     int fPID;
     bool fActive;
 
-    int fSessionID;
+    jack_uuid_t fSessionID;
     char fSessionCommand[JACK_SESSION_COMMAND_SIZE];
     jack_session_flags_t fSessionFlags;
 
-    JackClientControl(const char* name, int pid, int refnum, int uuid)
+    JackClientControl(const char* name, int pid, int refnum, jack_uuid_t uuid)
     {
         Init(name, pid, refnum, uuid);
     }
 
-    JackClientControl(const char* name)
+    JackClientControl(const char* name, jack_uuid_t uuid)
     {
-        Init(name, 0, -1, -1);
+        Init(name, 0, -1, uuid);
     }
 
     JackClientControl()
     {
-        Init("", 0, -1, -1);
+        Init("", 0, -1, JACK_UUID_EMPTY_INITIALIZER);
     }
 
-    void Init(const char* name, int pid, int refnum, int uuid)
+    void Init(const char* name, int pid, int refnum, jack_uuid_t uuid)
     {
         strcpy(fName, name);
-        for (int i = 0; i < kMaxNotification; i++)
+        for (int i = 0; i < kMaxNotification; i++) {
             fCallback[i] = false;
+        }
         // Always activated
         fCallback[kAddClient] = true;
         fCallback[kRemoveClient] = true;
