@@ -34,6 +34,7 @@ namespace Jack
 class JackLockedEngine;
 class JackGraphManager;
 struct JackEngineControl;
+class JackSlaveDriverInterface;
 
 /*!
 \brief The base interface for drivers.
@@ -51,16 +52,6 @@ class SERVER_EXPORT JackDriverInterface
 
         virtual int Open() = 0;
 
-        virtual int Open (bool capturing,
-                     bool playing,
-                     int inchannels,
-                     int outchannels,
-                     bool monitor,
-                     const char* capture_driver_name,
-                     const char* playback_driver_name,
-                     jack_nframes_t capture_latency,
-                     jack_nframes_t playback_latency) = 0;
-
         virtual int Open(jack_nframes_t buffer_size,
                          jack_nframes_t samplerate,
                          bool capturing,
@@ -87,14 +78,30 @@ class SERVER_EXPORT JackDriverInterface
         virtual int SetSampleRate(jack_nframes_t sample_rate) = 0;
 
         virtual int Process() = 0;
-        virtual int ProcessNull() = 0;
 
         virtual void SetMaster(bool onoff) = 0;
         virtual bool GetMaster() = 0;
+
         virtual void AddSlave(JackDriverInterface* slave) = 0;
         virtual void RemoveSlave(JackDriverInterface* slave) = 0;
+
         virtual std::list<JackDriverInterface*> GetSlaves() = 0;
-        virtual int ProcessSlaves() = 0;
+
+        // For "master" driver
+        virtual int ProcessReadSlaves() = 0;
+        virtual int ProcessWriteSlaves() = 0;
+
+        // For "slave" driver
+        virtual int ProcessRead() = 0;
+        virtual int ProcessWrite() = 0;
+
+        // For "slave" driver in "synchronous" mode
+        virtual int ProcessReadSync() = 0;
+        virtual int ProcessWriteSync() = 0;
+
+        // For "slave" driver in "asynchronous" mode
+        virtual int ProcessReadAsync() = 0;
+        virtual int ProcessWriteAsync() = 0;
 
         virtual bool IsRealTime() const = 0;
         virtual bool IsRunning() const = 0;
@@ -115,27 +122,47 @@ class SERVER_EXPORT JackDriverClientInterface : public JackDriverInterface, publ
 #define PlaybackDriverFlags static_cast<JackPortFlags>(JackPortIsInput | JackPortIsPhysical | JackPortIsTerminal)
 #define MonitorDriverFlags static_cast<JackPortFlags>(JackPortIsOutput)
 
+typedef std::list<std::pair<std::string, std::pair<std::string, std::string> > > driver_connections_list_t; // [type : (src, dst)]
+
 class SERVER_EXPORT JackDriver : public JackDriverClientInterface
 {
 
     protected:
 
-        char fCaptureDriverName[JACK_CLIENT_NAME_SIZE + 1];
-        char fPlaybackDriverName[JACK_CLIENT_NAME_SIZE + 1];
-        char fAliasName[JACK_CLIENT_NAME_SIZE + 1];
+        char fCaptureDriverName[JACK_CLIENT_NAME_SIZE+1];
+        char fPlaybackDriverName[JACK_CLIENT_NAME_SIZE+1];
+        char fAliasName[JACK_CLIENT_NAME_SIZE+1];
+
         jack_nframes_t fCaptureLatency;
         jack_nframes_t fPlaybackLatency;
+
+        int fCaptureChannels;
+        int fPlaybackChannels;
+
         jack_time_t fBeginDateUst;
         jack_time_t fEndDateUst;
         float fDelayedUsecs;
+
+        // Pointers to engine state
         JackLockedEngine* fEngine;
         JackGraphManager* fGraphManager;
         JackSynchro* fSynchroTable;
         JackEngineControl* fEngineControl;
         JackClientControl fClientControl;
+
         std::list<JackDriverInterface*> fSlaveList;
+
         bool fIsMaster;
         bool fIsRunning;
+        bool fWithMonitorPorts;
+
+        // Static tables since the actual number of ports may be changed by the real driver
+        // thus dynamic allocation is more difficult to handle
+        jack_port_id_t fCapturePortList[DRIVER_PORT_NUM];
+        jack_port_id_t fPlaybackPortList[DRIVER_PORT_NUM];
+        jack_port_id_t fMonitorPortList[DRIVER_PORT_NUM];
+
+        driver_connections_list_t fConnections;		// Connections list
 
         void CycleIncTime();
         void CycleTakeBeginTime();
@@ -148,10 +175,19 @@ class SERVER_EXPORT JackDriver : public JackDriverClientInterface
         void NotifySampleRate(jack_nframes_t sample_rate);                  // SampleRate notification sent by the driver
         void NotifyFailure(int code, const char* reason);                   // Failure notification sent by the driver
 
+        virtual void SaveConnections(int alias);
+        virtual void LoadConnections(int alias, bool full_name = true);
+        std::string MatchPortName(const char* name, const char** ports, int alias, const std::string& type);
+
+        virtual int StartSlaves();
+        virtual int StopSlaves();
+
+        virtual int ResumeRefNum();
+        virtual int SuspendRefNum();
+
     public:
 
         JackDriver(const char* name, const char* alias, JackLockedEngine* engine, JackSynchro* table);
-        JackDriver();
         virtual ~JackDriver();
 
         void SetMaster(bool onoff);
@@ -159,24 +195,14 @@ class SERVER_EXPORT JackDriver : public JackDriverClientInterface
 
         void AddSlave(JackDriverInterface* slave);
         void RemoveSlave(JackDriverInterface* slave);
+
         std::list<JackDriverInterface*> GetSlaves()
         {
             return fSlaveList;
         }
-        int ProcessSlaves();
 
         virtual int Open();
 
-        virtual int Open (bool capturing,
-                     bool playing,
-                     int inchannels,
-                     int outchannels,
-                     bool monitor,
-                     const char* capture_driver_name,
-                     const char* playback_driver_name,
-                     jack_nframes_t capture_latency,
-                     jack_nframes_t playback_latency);
-
         virtual int Open(jack_nframes_t buffer_size,
                          jack_nframes_t samplerate,
                          bool capturing,
@@ -188,10 +214,10 @@ class SERVER_EXPORT JackDriver : public JackDriverClientInterface
                          const char* playback_driver_name,
                          jack_nframes_t capture_latency,
                          jack_nframes_t playback_latency);
+
         virtual int Close();
 
         virtual int Process();
-        virtual int ProcessNull();
 
         virtual int Attach();
         virtual int Detach();
@@ -200,9 +226,23 @@ class SERVER_EXPORT JackDriver : public JackDriverClientInterface
         virtual int Write();
 
         virtual int Start();
-        virtual int StartSlaves();
         virtual int Stop();
-        virtual int StopSlaves();
+
+        // For "master" driver
+        int ProcessReadSlaves();
+        int ProcessWriteSlaves();
+
+        // For "slave" driver with typically decompose a given cycle in separated Read and Write parts.
+        virtual int ProcessRead();
+        virtual int ProcessWrite();
+
+        // For "slave" driver in "synchronous" mode
+        virtual int ProcessReadSync();
+        virtual int ProcessWriteSync();
+
+        // For "slave" driver in "asynchronous" mode
+        virtual int ProcessReadAsync();
+        virtual int ProcessWriteAsync();
 
         virtual bool IsFixedBufferSize();
         virtual int SetBufferSize(jack_nframes_t buffer_size);
