@@ -21,6 +21,7 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #include <fstream>
 #include <set>
 #include <assert.h>
+#include <ctype.h>
 
 #include "JackSystemDeps.h"
 #include "JackLockedEngine.h"
@@ -33,20 +34,28 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #include "JackChannel.h"
 #include "JackError.h"
 
+extern const char* JACK_METADATA_HARDWARE;
+extern const char* JACK_METADATA_PRETTY_NAME;
+
 namespace Jack
 {
 
 JackEngine::JackEngine(JackGraphManager* manager,
                        JackSynchro* table,
-                       JackEngineControl* control)
+                       JackEngineControl* control,
+                       char self_connect_mode)
+                    : JackLockAble(control->fServerName),
+                    fSignal(control->fServerName),
+                    fMetadata(true)
 {
     fGraphManager = manager;
     fSynchroTable = table;
     fEngineControl = control;
-    for (int i = 0; i < CLIENT_NUM; i++)
+    fSelfConnectMode = self_connect_mode;
+    for (int i = 0; i < CLIENT_NUM; i++) {
         fClientTable[i] = NULL;
+    }
     fLastSwitchUsecs = 0;
-    fMaxUUID = 0;
     fSessionPendingReplies = 0;
     fSessionTransaction = NULL;
     fSessionResult = NULL;
@@ -78,14 +87,13 @@ int JackEngine::Close()
         if (JackLoadableInternalClient* loadable_client = dynamic_cast<JackLoadableInternalClient*>(fClientTable[i])) {
             jack_log("JackEngine::Close loadable client = %s", loadable_client->GetClientControl()->fName);
             loadable_client->Close();
-            // Close does not delete the pointer for internal clients
             fClientTable[i] = NULL;
             delete loadable_client;
         } else if (JackExternalClient* external_client = dynamic_cast<JackExternalClient*>(fClientTable[i])) {
             jack_log("JackEngine::Close external client = %s", external_client->GetClientControl()->fName);
             external_client->Close();
-            // Close deletes the pointer for external clients
             fClientTable[i] = NULL;
+            delete external_client;
         }
     }
 
@@ -97,8 +105,9 @@ void JackEngine::NotifyQuit()
     fChannel.NotifyQuit();
 }
 
+
 //-----------------------------
-// Client ressource management
+// Client resource management
 //-----------------------------
 
 int JackEngine::AllocateRefnum()
@@ -112,18 +121,19 @@ int JackEngine::AllocateRefnum()
     return -1;
 }
 
-void JackEngine::ReleaseRefnum(int ref)
+void JackEngine::ReleaseRefnum(int refnum)
 {
-    fClientTable[ref] = NULL;
+    fClientTable[refnum] = NULL;
 
     if (fEngineControl->fTemporary) {
         int i;
         for (i = fEngineControl->fDriverNum; i < CLIENT_NUM; i++) {
-            if (fClientTable[i])
+            if (fClientTable[i]) {
                 break;
+            }
         }
         if (i == CLIENT_NUM) {
-            // last client and temporay case: quit the server
+            // Last client and temporary case: quit the server
             jack_log("JackEngine::ReleaseRefnum server quit");
             fEngineControl->fTemporary = false;
             throw JackTemporaryException();
@@ -138,17 +148,17 @@ void JackEngine::ReleaseRefnum(int ref)
 void JackEngine::ProcessNext(jack_time_t cur_cycle_begin)
 {
     fLastSwitchUsecs = cur_cycle_begin;
-    if (fGraphManager->RunNextGraph())  { // True if the graph actually switched to a new state
+    if (fGraphManager->RunNextGraph())  {   // True if the graph actually switched to a new state
         fChannel.Notify(ALL_CLIENTS, kGraphOrderCallback, 0);
-        //NotifyGraphReorder();
     }
-    fSignal.Signal();                   // Signal for threads waiting for next cycle
+    fSignal.Signal();                       // Signal for threads waiting for next cycle
 }
 
 void JackEngine::ProcessCurrent(jack_time_t cur_cycle_begin)
 {
-    if (cur_cycle_begin < fLastSwitchUsecs + 2 * fEngineControl->fPeriodUsecs) // Signal XRun only for the first failing cycle
+    if (cur_cycle_begin < fLastSwitchUsecs + 2 * fEngineControl->fPeriodUsecs) { // Signal XRun only for the first failing cycle
         CheckXRun(cur_cycle_begin);
+    }
     fGraphManager->RunCurrentGraph();
 }
 
@@ -186,6 +196,22 @@ Client that finish *after* the callback date are considered late even if their o
 correctly mixed in the time window: callbackUsecs <==> Read <==> Write.
 */
 
+static const char* State2String(jack_client_state_t state)
+{
+    switch (state) {
+        case NotTriggered:
+            return "NotTriggered";
+        case Triggered:
+            return "Triggered";
+        case Running:
+            return "Running";
+        case Finished:
+            return "Finished";
+        default:
+            return "";
+    }
+}
+
 void JackEngine::CheckXRun(jack_time_t callback_usecs)  // REVOIR les conditions de fin
 {
     for (int i = fEngineControl->fDriverNum; i < CLIENT_NUM; i++) {
@@ -196,15 +222,13 @@ void JackEngine::CheckXRun(jack_time_t callback_usecs)  // REVOIR les conditions
             jack_time_t finished_date = timing->fFinishedAt;
 
             if (status != NotTriggered && status != Finished) {
-                jack_error("JackEngine::XRun: client = %s was not run: state = %ld", client->GetClientControl()->fName, status);
+                jack_error("JackEngine::XRun: client = %s was not finished, state = %s", client->GetClientControl()->fName, State2String(status));
                 fChannel.Notify(ALL_CLIENTS, kXRunCallback, 0);  // Notify all clients
-                //NotifyXRun(ALL_CLIENTS);
             }
 
             if (status == Finished && (long)(finished_date - callback_usecs) > 0) {
                 jack_error("JackEngine::XRun: client %s finished after current callback", client->GetClientControl()->fName);
                 fChannel.Notify(ALL_CLIENTS, kXRunCallback, 0);  // Notify all clients
-                //NotifyXRun(ALL_CLIENTS);
             }
         }
     }
@@ -219,15 +243,15 @@ int JackEngine::ComputeTotalLatencies()
     fGraphManager->TopologicalSort(sorted);
 
     /* iterate over all clients in graph order, and emit
-	 * capture latency callback.
-	 */
+     * capture latency callback.
+     */
 
     for (it = sorted.begin(); it != sorted.end(); it++) {
         NotifyClient(*it, kLatencyCallback, true, "", 0, 0);
     }
 
     /* now issue playback latency callbacks in reverse graph order.
-	 */
+     */
     for (rit = sorted.rbegin(); rit != sorted.rend(); rit++) {
         NotifyClient(*rit, kLatencyCallback, true, "", 1, 0);
     }
@@ -235,30 +259,63 @@ int JackEngine::ComputeTotalLatencies()
     return 0;
 }
 
+//--------------
+// Metadata API
+//--------------
+
+int JackEngine::PropertyChangeNotify(jack_uuid_t subject, const char* key, jack_property_change_t change)
+{
+    jack_log("JackEngine::PropertyChangeNotify: subject = %x key = %s change = %x", subject, key, change);
+
+    for (int i = 0; i < CLIENT_NUM; i++) {
+        JackClientInterface* client = fClientTable[i];
+        if (client) {
+            char buf[JACK_UUID_STRING_SIZE];
+            jack_uuid_unparse(subject, buf);
+            client->ClientNotify(i, buf, kPropertyChangeCallback, false, key, change, 0);
+        }
+    }
+
+    return 0;
+}
+
 //---------------
 // Notifications
 //---------------
 
+int JackEngine::ClientNotify(JackClientInterface* client, int refnum, const char* name, int notify, int sync, const char* message, int value1, int value2)
+{
+    // Check if notification is needed
+    if (!client->GetClientControl()->fCallback[notify]) {
+        jack_log("JackEngine::ClientNotify: no callback for notification = %ld", notify);
+        return 0;
+    }
+
+    int res1;
+
+    // External client
+    if (dynamic_cast<JackExternalClient*>(client)) {
+       res1 = client->ClientNotify(refnum, name, notify, sync, message, value1, value2);
+    // Important for internal client : unlock before calling the notification callbacks
+    } else {
+        bool res2 = Unlock();
+        res1 = client->ClientNotify(refnum, name, notify, sync, message, value1, value2);
+        if (res2) {
+            Lock();
+        }
+    }
+
+    if (res1 < 0) {
+        jack_error("ClientNotify fails name = %s notification = %ld val1 = %ld val2 = %ld", name, notify, value1, value2);
+    }
+    return res1;
+}
+
 void JackEngine::NotifyClient(int refnum, int event, int sync, const char* message, int value1, int value2)
 {
     JackClientInterface* client = fClientTable[refnum];
-
-    // The client may be notified by the RT thread while closing
     if (client) {
-
-        if (client->GetClientControl()->fCallback[event]) {
-            /*
-                Important for internal clients : unlock before calling the notification callbacks.
-            */
-            bool res = fMutex.Unlock();
-            if (client->ClientNotify(refnum, client->GetClientControl()->fName, event, sync, message, value1, value2) < 0)
-                jack_error("NotifyClient fails name = %s event = %ld val1 = %ld val2 = %ld", client->GetClientControl()->fName, event, value1, value2);
-            if (res)
-                fMutex.Lock();
-
-        } else {
-            jack_log("JackEngine::NotifyClient: no callback for event = %ld", event);
-        }
+        ClientNotify(client, refnum, client->GetClientControl()->fName, event, sync, message, value1, value2);
     }
 }
 
@@ -269,19 +326,21 @@ void JackEngine::NotifyClients(int event, int sync, const char* message, int val
     }
 }
 
-int JackEngine::NotifyAddClient(JackClientInterface* new_client, const char* name, int refnum)
+int JackEngine::NotifyAddClient(JackClientInterface* new_client, const char* new_name, int refnum)
 {
-    jack_log("JackEngine::NotifyAddClient: name = %s", name);
+    jack_log("JackEngine::NotifyAddClient: name = %s", new_name);
+
     // Notify existing clients of the new client and new client of existing clients.
     for (int i = 0; i < CLIENT_NUM; i++) {
         JackClientInterface* old_client = fClientTable[i];
-        if (old_client) {
-            if (old_client->ClientNotify(refnum, name, kAddClient, true, "", 0, 0) < 0) {
-                jack_error("NotifyAddClient old_client fails name = %s", old_client->GetClientControl()->fName);
-                return -1;
+        if (old_client && old_client != new_client) {
+            char* old_name = old_client->GetClientControl()->fName;
+            if (ClientNotify(old_client, refnum, new_name, kAddClient, false, "", 0, 0) < 0) {
+                jack_error("NotifyAddClient old_client fails name = %s", old_name);
+                // Not considered as a failure...
             }
-            if (new_client->ClientNotify(i, old_client->GetClientControl()->fName, kAddClient, true, "", 0, 0) < 0) {
-                jack_error("NotifyAddClient new_client fails name = %s", name);
+            if (ClientNotify(new_client, i, old_name, kAddClient, true, "", 0, 0) < 0) {
+                jack_error("NotifyAddClient new_client fails name = %s", new_name);
                 return -1;
             }
         }
@@ -292,25 +351,23 @@ int JackEngine::NotifyAddClient(JackClientInterface* new_client, const char* nam
 
 void JackEngine::NotifyRemoveClient(const char* name, int refnum)
 {
-    // Notify existing clients (including the one beeing suppressed) of the removed client
+    // Notify existing clients (including the one being suppressed) of the removed client
     for (int i = 0; i < CLIENT_NUM; i++) {
         JackClientInterface* client = fClientTable[i];
         if (client) {
-            client->ClientNotify(refnum, name, kRemoveClient, true, "",0, 0);
+            ClientNotify(client, refnum, name, kRemoveClient, false, "", 0, 0);
         }
     }
 }
 
 // Coming from the driver
-void JackEngine::NotifyXRun(jack_time_t callback_usecs, float delayed_usecs)
+void JackEngine::NotifyDriverXRun()
 {
     // Use the audio thread => request thread communication channel
-    fEngineControl->NotifyXRun(callback_usecs, delayed_usecs);
     fChannel.Notify(ALL_CLIENTS, kXRunCallback, 0);
-    //NotifyXRun(ALL_CLIENTS);
 }
 
-void JackEngine::NotifyXRun(int refnum)
+void JackEngine::NotifyClientXRun(int refnum)
 {
     if (refnum == ALL_CLIENTS) {
         NotifyClients(kXRunCallback, false, "", 0, 0);
@@ -321,8 +378,8 @@ void JackEngine::NotifyXRun(int refnum)
 
 void JackEngine::NotifyGraphReorder()
 {
-    NotifyClients(kGraphOrderCallback, false, "", 0, 0);
     ComputeTotalLatencies();
+    NotifyClients(kGraphOrderCallback, false, "", 0, 0);
 }
 
 void JackEngine::NotifyBufferSize(jack_nframes_t buffer_size)
@@ -381,6 +438,7 @@ void JackEngine::NotifyActivate(int refnum)
 int JackEngine::GetInternalClientName(int refnum, char* name_res)
 {
     JackClientInterface* client = fClientTable[refnum];
+    assert(client);
     strncpy(name_res, client->GetClientControl()->fName, JACK_CLIENT_NAME_SIZE);
     return 0;
 }
@@ -421,13 +479,13 @@ int JackEngine::InternalClientUnload(int refnum, int* status)
 // Client management
 //-------------------
 
-int JackEngine::ClientCheck(const char* name, int uuid, char* name_res, int protocol, int options, int* status)
+int JackEngine::ClientCheck(const char* name, jack_uuid_t uuid, char* name_res, int protocol, int options, int* status)
 {
     // Clear status
     *status = 0;
     strcpy(name_res, name);
 
-    jack_log("Check protocol client %ld server = %ld", protocol, JACK_PROTOCOL_VERSION);
+    jack_log("Check protocol client = %ld server = %ld", protocol, JACK_PROTOCOL_VERSION);
 
     if (protocol != JACK_PROTOCOL_VERSION) {
         *status |= (JackFailure | JackVersionError);
@@ -495,32 +553,31 @@ bool JackEngine::ClientCheckName(const char* name)
 {
     for (int i = 0; i < CLIENT_NUM; i++) {
         JackClientInterface* client = fClientTable[i];
-        if (client && (strcmp(client->GetClientControl()->fName, name) == 0))
+        if (client && (strcmp(client->GetClientControl()->fName, name) == 0)) {
             return true;
+        }
     }
 
     for (std::map<int,std::string>::iterator i = fReservationMap.begin(); i != fReservationMap.end(); i++) {
-        if (i->second == name)
+        if (i->second == name) {
             return true;
+        }
     }
 
     return false;
 }
 
-int JackEngine::GetNewUUID()
+void JackEngine::EnsureUUID(jack_uuid_t uuid)
 {
-    return fMaxUUID++;
-}
-
-void JackEngine::EnsureUUID(int uuid)
-{
-    if (uuid > fMaxUUID)
-        fMaxUUID = uuid+1;
+    if (jack_uuid_empty(uuid))
+        return;
 
     for (int i = 0; i < CLIENT_NUM; i++) {
         JackClientInterface* client = fClientTable[i];
-        if (client && (client->GetClientControl()->fSessionID == uuid)) {
-            client->GetClientControl()->fSessionID = GetNewUUID();
+        if (client && jack_uuid_compare(client->GetClientControl()->fSessionID, uuid) == 0) {
+            // FIXME? this code does nothing, but jack1 has it like this too..
+            jack_uuid_clear (&uuid);
+            // client->GetClientControl()->fSessionID = jack_client_uuid_generate();
         }
     }
 }
@@ -529,8 +586,9 @@ int JackEngine::GetClientPID(const char* name)
 {
     for (int i = 0; i < CLIENT_NUM; i++) {
         JackClientInterface* client = fClientTable[i];
-        if (client && (strcmp(client->GetClientControl()->fName, name) == 0))
+        if (client && (strcmp(client->GetClientControl()->fName, name) == 0)) {
             return client->GetClientControl()->fPID;
+        }
     }
 
     return 0;
@@ -540,20 +598,21 @@ int JackEngine::GetClientRefNum(const char* name)
 {
     for (int i = 0; i < CLIENT_NUM; i++) {
         JackClientInterface* client = fClientTable[i];
-        if (client && (strcmp(client->GetClientControl()->fName, name) == 0))
+        if (client && (strcmp(client->GetClientControl()->fName, name) == 0)) {
             return client->GetClientControl()->fRefNum;
+        }
     }
 
     return -1;
 }
 
 // Used for external clients
-int JackEngine::ClientExternalOpen(const char* name, int pid, int uuid, int* ref, int* shared_engine, int* shared_client, int* shared_graph_manager)
+int JackEngine::ClientExternalOpen(const char* name, int pid, jack_uuid_t uuid, int* ref, int* shared_engine, int* shared_client, int* shared_graph_manager)
 {
     char real_name[JACK_CLIENT_NAME_SIZE + 1];
 
-    if (uuid < 0) {
-        uuid = GetNewUUID();
+    if (jack_uuid_empty(uuid)) {
+        uuid = jack_client_uuid_generate();
         strncpy(real_name, name, JACK_CLIENT_NAME_SIZE);
     } else {
         std::map<int, std::string>::iterator res = fReservationMap.find(uuid);
@@ -563,11 +622,10 @@ int JackEngine::ClientExternalOpen(const char* name, int pid, int uuid, int* ref
         } else {
             strncpy(real_name, name, JACK_CLIENT_NAME_SIZE);
         }
-
         EnsureUUID(uuid);
     }
 
-    jack_log("JackEngine::ClientExternalOpen: uuid = %d, name = %s ", uuid, real_name);
+    jack_log("JackEngine::ClientExternalOpen: uuid = %d, name = %s", uuid, real_name);
 
     int refnum = AllocateRefnum();
     if (refnum < 0) {
@@ -662,9 +720,10 @@ error:
 // Used for external clients
 int JackEngine::ClientExternalClose(int refnum)
 {
+    jack_log("JackEngine::ClientExternalClose ref = %ld", refnum);
     JackClientInterface* client = fClientTable[refnum];
-    fEngineControl->fTransport.ResetTimebase(refnum);
-    int res = ClientCloseAux(refnum, client, true);
+    assert(client);
+    int res = ClientCloseAux(refnum, true);
     client->Close();
     delete client;
     return res;
@@ -673,25 +732,31 @@ int JackEngine::ClientExternalClose(int refnum)
 // Used for server internal clients or drivers when the RT thread is stopped
 int JackEngine::ClientInternalClose(int refnum, bool wait)
 {
-    JackClientInterface* client = fClientTable[refnum];
-    return ClientCloseAux(refnum, client, wait);
+    jack_log("JackEngine::ClientInternalClose ref = %ld", refnum);
+    return ClientCloseAux(refnum, wait);
 }
 
-int JackEngine::ClientCloseAux(int refnum, JackClientInterface* client, bool wait)
+int JackEngine::ClientCloseAux(int refnum, bool wait)
 {
     jack_log("JackEngine::ClientCloseAux ref = %ld", refnum);
 
+    JackClientInterface* client = fClientTable[refnum];
+    fEngineControl->fTransport.ResetTimebase(refnum);
+
+    jack_uuid_t uuid = JACK_UUID_EMPTY_INITIALIZER;
+    jack_uuid_copy (&uuid, client->GetClientControl()->fSessionID);
+
     // Unregister all ports ==> notifications are sent
     jack_int_t ports[PORT_NUM_FOR_CLIENT];
     int i;
 
     fGraphManager->GetInputPorts(refnum, ports);
-    for (i = 0; (i < PORT_NUM_FOR_CLIENT) && (ports[i] != EMPTY) ; i++) {
+    for (i = 0; (i < PORT_NUM_FOR_CLIENT) && (ports[i] != EMPTY); i++) {
         PortUnRegister(refnum, ports[i]);
     }
 
     fGraphManager->GetOutputPorts(refnum, ports);
-    for (i = 0; (i < PORT_NUM_FOR_CLIENT) && (ports[i] != EMPTY) ; i++) {
+    for (i = 0; (i < PORT_NUM_FOR_CLIENT) && (ports[i] != EMPTY); i++) {
         PortUnRegister(refnum, ports[i]);
     }
 
@@ -708,8 +773,15 @@ int JackEngine::ClientCloseAux(int refnum, JackClientInterface* client, bool wai
         }
     }
 
+    if (fMetadata.RemoveProperties(NULL, uuid) > 0) {
+        /* have to do the notification ourselves, since the client argument
+          to fMetadata->RemoveProperties() was NULL
+        */
+        PropertyChangeNotify(uuid, NULL, PropertyDeleted);
+    }
+
     // Notify running clients
-    NotifyRemoveClient(client->GetClientControl()->fName, client->GetClientControl()->fRefNum);
+    NotifyRemoveClient(client->GetClientControl()->fName, refnum);
 
     // Cleanup...
     fSynchroTable[refnum].Destroy();
@@ -722,8 +794,9 @@ int JackEngine::ClientActivate(int refnum, bool is_real_time)
     JackClientInterface* client = fClientTable[refnum];
     jack_log("JackEngine::ClientActivate ref = %ld name = %s", refnum, client->GetClientControl()->fName);
 
-    if (is_real_time)
+    if (is_real_time) {
         fGraphManager->Activate(refnum);
+    }
 
     // Wait for graph state change to be effective
     if (!fSignal.LockedTimedWait(fEngineControl->fTimeOutUsecs * 10)) {
@@ -763,10 +836,10 @@ int JackEngine::ClientDeactivate(int refnum)
 
     // First disconnect all ports
     for (int i = 0; (i < PORT_NUM_FOR_CLIENT) && (input_ports[i] != EMPTY); i++) {
-        PortDisconnect(refnum, input_ports[i], ALL_PORTS);
+        PortDisconnect(-1, input_ports[i], ALL_PORTS);
     }
     for (int i = 0; (i < PORT_NUM_FOR_CLIENT) && (output_ports[i] != EMPTY); i++) {
-        PortDisconnect(refnum, output_ports[i], ALL_PORTS);
+        PortDisconnect(-1, output_ports[i], ALL_PORTS);
     }
 
     // Then issue port registration notification
@@ -789,6 +862,17 @@ int JackEngine::ClientDeactivate(int refnum)
     }
 }
 
+void JackEngine::ClientKill(int refnum)
+{
+    jack_log("JackEngine::ClientKill ref = %ld", refnum);
+    if (ClientDeactivate(refnum) < 0) {
+        jack_error("JackEngine::ClientKill ref = %ld cannot be removed from the graph !!", refnum);
+    }
+    if (ClientExternalClose(refnum) < 0) {
+        jack_error("JackEngine::ClientKill ref = %ld cannot be closed", refnum);
+    }
+}
+
 //-----------------
 // Port management
 //-----------------
@@ -804,10 +888,12 @@ int JackEngine::PortRegister(int refnum, const char* name, const char *type, uns
         return -1;
     }
 
+    // buffer_size is actually ignored...
     *port_index = fGraphManager->AllocatePort(refnum, name, type, (JackPortFlags)flags, fEngineControl->fBufferSize);
     if (*port_index != NO_PORT) {
-        if (client->GetClientControl()->fActive)
+        if (client->GetClientControl()->fActive) {
             NotifyPortRegistation(*port_index, true);
+        }
         return 0;
     } else {
         return -1;
@@ -818,22 +904,77 @@ int JackEngine::PortUnRegister(int refnum, jack_port_id_t port_index)
 {
     jack_log("JackEngine::PortUnRegister ref = %ld port_index = %ld", refnum, port_index);
     JackClientInterface* client = fClientTable[refnum];
+    assert(client);
 
     // Disconnect port ==> notification is sent
-    PortDisconnect(refnum, port_index, ALL_PORTS);
+    PortDisconnect(-1, port_index, ALL_PORTS);
 
     if (fGraphManager->ReleasePort(refnum, port_index) == 0) {
-        if (client->GetClientControl()->fActive)
+        const jack_uuid_t uuid = jack_port_uuid_generate(port_index);
+        if (!jack_uuid_empty(uuid))
+        {
+            if (fMetadata.RemoveProperties(NULL, uuid) > 0) {
+                /* have to do the notification ourselves, since the client argument
+                   to fMetadata->RemoveProperties() was NULL
+                */
+                PropertyChangeNotify(uuid, NULL, PropertyDeleted);
+            }
+        }
+
+        if (client->GetClientControl()->fActive) {
             NotifyPortRegistation(port_index, false);
+        }
         return 0;
     } else {
         return -1;
     }
 }
 
+// this check is to prevent apps to self connect to other apps
+// TODO: make this work with multiple clients per app
+int JackEngine::CheckPortsConnect(int refnum, jack_port_id_t src, jack_port_id_t dst)
+{
+    if (fSelfConnectMode == ' ') return 1;
+
+    JackPort* src_port = fGraphManager->GetPort(src);
+    JackPort* dst_port = fGraphManager->GetPort(dst);
+
+    jack_log("JackEngine::CheckPortsConnect(ref = %d, src = %d, dst = %d)", refnum, src_port->GetRefNum(), dst_port->GetRefNum());
+
+    //jack_log("%s -> %s", src_port->GetName(), dst_port->GetName());
+    //jack_log("mode = '%c'", fSelfConnectMode);
+
+    int src_self = src_port->GetRefNum() == refnum ? 1 : 0;
+    int dst_self = dst_port->GetRefNum() == refnum ? 1 : 0;
+
+    //jack_log("src_self is %s", src_self ? "true" : "false");
+    //jack_log("dst_self is %s", dst_self ? "true" : "false");
+
+    // 0 means client is connecting other client ports (control app patchbay functionality)
+    // 1 means client is connecting its own port to port of other client (e.g. self connecting into "system" client)
+    // 2 means client is connecting its own ports (for app internal functionality)
+    int sum = src_self + dst_self;
+    //jack_log("sum = %d", sum);
+    if (sum == 0) return 1;
+    char lmode = tolower(fSelfConnectMode);
+    //jack_log("lmode = '%c'", lmode);
+    if (sum == 2 && lmode == 'e') return 1;
+    bool fail = lmode != fSelfConnectMode; // fail modes are upper case
+    //jack_log("fail = %d", (int)fail);
+
+    jack_info(
+        "%s port self connect request%s (%s -> %s)",
+        fail ? "rejecting" : "ignoring",
+        sum == 1 ? " to external port" : "",
+        src_port->GetName(),
+        dst_port->GetName());
+
+    return fail ? -1 : 0;
+}
+
 int JackEngine::PortConnect(int refnum, const char* src, const char* dst)
 {
-    jack_log("JackEngine::PortConnect src = %s dst = %s", src, dst);
+    jack_log("JackEngine::PortConnect ref = %d src = %s dst = %s", refnum, src, dst);
     jack_port_id_t port_src, port_dst;
 
     return (fGraphManager->GetTwoPorts(src, dst, &port_src, &port_dst) < 0)
@@ -843,12 +984,13 @@ int JackEngine::PortConnect(int refnum, const char* src, const char* dst)
 
 int JackEngine::PortConnect(int refnum, jack_port_id_t src, jack_port_id_t dst)
 {
-    jack_log("JackEngine::PortConnect src = %d dst = %d", src, dst);
+    jack_log("JackEngine::PortConnect ref = %d src = %d dst = %d", refnum, src, dst);
     JackClientInterface* client;
     int ref;
 
-    if (fGraphManager->CheckPorts(src, dst) < 0)
+    if (fGraphManager->CheckPorts(src, dst) < 0) {
         return -1;
+    }
 
     ref = fGraphManager->GetOutputRefNum(src);
     assert(ref >= 0);
@@ -870,15 +1012,21 @@ int JackEngine::PortConnect(int refnum, jack_port_id_t src, jack_port_id_t dst)
         return -1;
     }
 
-    int res = fGraphManager->Connect(src, dst);
-    if (res == 0)
+    int res = CheckPortsConnect(refnum, src, dst);
+    if (res != 1) {
+        return res;
+    }
+
+    res = fGraphManager->Connect(src, dst);
+    if (res == 0) {
         NotifyPortConnect(src, dst, true);
+    }
     return res;
 }
 
 int JackEngine::PortDisconnect(int refnum, const char* src, const char* dst)
 {
-    jack_log("JackEngine::PortDisconnect src = %s dst = %s", src, dst);
+    jack_log("JackEngine::PortDisconnect ref = %d src = %s dst = %s", refnum, src, dst);
     jack_port_id_t port_src, port_dst;
 
     return (fGraphManager->GetTwoPorts(src, dst, &port_src, &port_dst) < 0)
@@ -888,7 +1036,7 @@ int JackEngine::PortDisconnect(int refnum, const char* src, const char* dst)
 
 int JackEngine::PortDisconnect(int refnum, jack_port_id_t src, jack_port_id_t dst)
 {
-    jack_log("JackEngine::PortDisconnect src = %d dst = %d", src, dst);
+    jack_log("JackEngine::PortDisconnect ref = %d src = %d dst = %d", refnum, src, dst);
 
     if (dst == ALL_PORTS) {
 
@@ -896,59 +1044,85 @@ int JackEngine::PortDisconnect(int refnum, jack_port_id_t src, jack_port_id_t ds
         fGraphManager->GetConnections(src, connections);
 
         JackPort* port = fGraphManager->GetPort(src);
-        int ret = 0;
+        int res = 0;
         if (port->GetFlags() & JackPortIsOutput) {
             for (int i = 0; (i < CONNECTION_NUM_FOR_PORT) && (connections[i] != EMPTY); i++) {
                 if (PortDisconnect(refnum, src, connections[i]) != 0) {
-                    ret = -1;
+                    res = -1;
                 }
             }
         } else {
             for (int i = 0; (i < CONNECTION_NUM_FOR_PORT) && (connections[i] != EMPTY); i++) {
                 if (PortDisconnect(refnum, connections[i], src) != 0) {
-                    ret = -1;
+                    res = -1;
                 }
             }
         }
 
-        return ret;
-    } else if (fGraphManager->CheckPorts(src, dst) < 0) {
-        return -1;
-    } else if (fGraphManager->Disconnect(src, dst) == 0) {
-        // Notifications
-        NotifyPortConnect(src, dst, false);
-        return 0;
-    } else {
+        return res;
+    }
+
+    if (fGraphManager->CheckPorts(src, dst) < 0) {
         return -1;
     }
+
+    int res = CheckPortsConnect(refnum, src, dst);
+    if (res != 1) {
+        return res;
+    }
+
+    res = fGraphManager->Disconnect(src, dst);
+    if (res == 0)
+        NotifyPortConnect(src, dst, false);
+    return res;
 }
 
 int JackEngine::PortRename(int refnum, jack_port_id_t port, const char* name)
 {
-    char old_name[JACK_CLIENT_NAME_SIZE + JACK_PORT_NAME_SIZE];
+    char old_name[REAL_JACK_PORT_NAME_SIZE+1];
     strcpy(old_name, fGraphManager->GetPort(port)->GetName());
     fGraphManager->GetPort(port)->SetName(name);
     NotifyPortRename(port, old_name);
     return 0;
 }
 
+int JackEngine::PortSetDefaultMetadata(jack_port_id_t port, const char* pretty_name)
+{
+    static const char* type = "text/plain";
+    jack_uuid_t uuid = jack_port_uuid_generate(port);
+
+    int res = fMetadata.SetProperty(NULL, uuid, JACK_METADATA_HARDWARE, pretty_name, type);
+    if (res == -1) {
+        return -1;
+    }
+
+    char *v, *t;
+    res = fMetadata.GetProperty(uuid, JACK_METADATA_PRETTY_NAME, &v, &t);
+    if (res == -1) {
+        res = fMetadata.SetProperty(NULL, uuid, JACK_METADATA_PRETTY_NAME, pretty_name, type);
+    }
+
+    return res;
+}
+
 //--------------------
 // Session management
 //--------------------
 
-void JackEngine::SessionNotify(int refnum, const char *target, jack_session_event_type_t type, const char *path, JackChannelTransaction *socket)
+void JackEngine::SessionNotify(int refnum, const char *target, jack_session_event_type_t type, const char *path, detail::JackChannelTransactionInterface *socket, JackSessionNotifyResult** result)
 {
     if (fSessionPendingReplies != 0) {
         JackSessionNotifyResult res(-1);
         res.Write(socket);
         jack_log("JackEngine::SessionNotify ... busy");
+        if (result != NULL) *result = NULL;
         return;
     }
 
     for (int i = 0; i < CLIENT_NUM; i++) {
         JackClientInterface* client = fClientTable[i];
-        if (client && (client->GetClientControl()->fSessionID < 0)) {
-            client->GetClientControl()->fSessionID = GetNewUUID();
+        if (client && jack_uuid_empty(client->GetClientControl()->fSessionID)) {
+            client->GetClientControl()->fSessionID = jack_client_uuid_generate();
         }
     }
     fSessionResult = new JackSessionNotifyResult();
@@ -965,41 +1139,47 @@ void JackEngine::SessionNotify(int refnum, const char *target, jack_session_even
             }
 
             char path_buf[JACK_PORT_NAME_SIZE];
-            snprintf( path_buf, sizeof(path_buf), "%s%s%c", path, client->GetClientControl()->fName, DIR_SEPARATOR );
+            if (path[strlen(path) - 1] == DIR_SEPARATOR) {
+               snprintf(path_buf, sizeof path_buf, "%s%s%c", path, client->GetClientControl()->fName, DIR_SEPARATOR);
+            } else {
+               snprintf(path_buf, sizeof path_buf, "%s%c%s%c", path, DIR_SEPARATOR, client->GetClientControl()->fName, DIR_SEPARATOR);
+            }
 
             int res = JackTools::MkDir(path_buf);
-            if (res)
-                jack_error( "JackEngine::SessionNotify: can not create session directory '%s'", path_buf );
+            if (res) jack_error("JackEngine::SessionNotify: can not create session directory '%s'", path_buf);
 
-            int result = client->ClientNotify(i, client->GetClientControl()->fName, kSessionCallback, true, path_buf, (int) type, 0);
+            int result = client->ClientNotify(i, client->GetClientControl()->fName, kSessionCallback, true, path_buf, (int)type, 0);
 
-            if (result == 2) {
+            if (result == kPendingSessionReply) {
                 fSessionPendingReplies += 1;
-            } else if (result == 1) {
-                char uuid_buf[JACK_UUID_SIZE];
-                snprintf( uuid_buf, sizeof(uuid_buf), "%d", client->GetClientControl()->fSessionID );
-                fSessionResult->fCommandList.push_back( JackSessionCommand( uuid_buf,
-                                                                            client->GetClientControl()->fName,
-                                                                            client->GetClientControl()->fSessionCommand,
-                                                                            client->GetClientControl()->fSessionFlags ));
+            } else if (result == kImmediateSessionReply) {
+                char uuid_buf[JACK_UUID_STRING_SIZE];
+                jack_uuid_unparse(client->GetClientControl()->fSessionID, uuid_buf);
+                fSessionResult->fCommandList.push_back(JackSessionCommand(uuid_buf,
+                                                                        client->GetClientControl()->fName,
+                                                                        client->GetClientControl()->fSessionCommand,
+                                                                        client->GetClientControl()->fSessionFlags));
             }
         }
     }
 
+    if (result != NULL) *result = fSessionResult;
+
     if (fSessionPendingReplies == 0) {
         fSessionResult->Write(socket);
-        delete fSessionResult;
+        if (result == NULL) delete fSessionResult;
         fSessionResult = NULL;
     } else {
         fSessionTransaction = socket;
     }
 }
 
-void JackEngine::SessionReply(int refnum)
+int JackEngine::SessionReply(int refnum)
 {
     JackClientInterface* client = fClientTable[refnum];
-    char uuid_buf[JACK_UUID_SIZE];
-    snprintf( uuid_buf, sizeof(uuid_buf), "%d", client->GetClientControl()->fSessionID);
+    assert(client);
+    char uuid_buf[JACK_UUID_STRING_SIZE];
+    jack_uuid_unparse(client->GetClientControl()->fSessionID, uuid_buf);
     fSessionResult->fCommandList.push_back(JackSessionCommand(uuid_buf,
                                                             client->GetClientControl()->fName,
                                                             client->GetClientControl()->fSessionCommand,
@@ -1008,75 +1188,85 @@ void JackEngine::SessionReply(int refnum)
 
     if (fSessionPendingReplies == 0) {
         fSessionResult->Write(fSessionTransaction);
-        delete fSessionResult;
+        if (fSessionTransaction != NULL) {
+            delete fSessionResult;
+        }
         fSessionResult = NULL;
     }
+
+    return 0;
 }
 
-void JackEngine::GetUUIDForClientName(const char *client_name, char *uuid_res, int *result)
+int JackEngine::GetUUIDForClientName(const char *client_name, char *uuid_res)
 {
     for (int i = 0; i < CLIENT_NUM; i++) {
         JackClientInterface* client = fClientTable[i];
 
         if (client && (strcmp(client_name, client->GetClientControl()->fName) == 0)) {
-            snprintf(uuid_res, JACK_UUID_SIZE, "%d", client->GetClientControl()->fSessionID);
-            *result = 0;
-            return;
+            jack_uuid_unparse(client->GetClientControl()->fSessionID, uuid_res);
+            return 0;
         }
     }
     // Did not find name.
-    *result = -1;
+    return -1;
 }
 
-void JackEngine::GetClientNameForUUID(const char *uuid, char *name_res, int *result)
+int JackEngine::GetClientNameForUUID(const char *uuid_buf, char *name_res)
 {
+    jack_uuid_t uuid;
+    if (jack_uuid_parse(uuid_buf, &uuid) != 0)
+        return -1;
+
     for (int i = 0; i < CLIENT_NUM; i++) {
         JackClientInterface* client = fClientTable[i];
 
-        if (!client)
+        if (!client) {
             continue;
+        }
 
-        char uuid_buf[JACK_UUID_SIZE];
-        snprintf(uuid_buf, JACK_UUID_SIZE, "%d", client->GetClientControl()->fSessionID);
-
-        if (strcmp(uuid,uuid_buf) == 0) {
+        if (jack_uuid_compare(client->GetClientControl()->fSessionID, uuid) == 0) {
             strncpy(name_res, client->GetClientControl()->fName, JACK_CLIENT_NAME_SIZE);
-            *result = 0;
-            return;
+            return 0;
         }
     }
     // Did not find uuid.
-    *result = -1;
+    return -1;
 }
 
-void JackEngine::ReserveClientName(const char *name, const char *uuid, int *result)
+int JackEngine::ReserveClientName(const char *name, const char *uuidstr)
 {
-    jack_log("JackEngine::ReserveClientName ( name = %s, uuid = %s )", name, uuid);
+    jack_log("JackEngine::ReserveClientName ( name = %s, uuid = %s )", name, uuidstr);
 
     if (ClientCheckName(name)) {
-        *result = -1;
         jack_log("name already taken");
-        return;
+        return -1;
     }
 
-    EnsureUUID(atoi(uuid));
-    fReservationMap[atoi(uuid)] = name;
-    *result = 0;
+    jack_uuid_t uuid;
+    if (jack_uuid_parse(uuidstr, &uuid) != 0) {
+        jack_error("JackEngine::ReserveClientName invalid uuid %s", uuidstr);
+        return -1;
+    }
+
+    EnsureUUID(uuid);
+    fReservationMap[uuid] = name;
+    return 0;
 }
 
-void JackEngine::ClientHasSessionCallbackRequest(const char *name, int *result)
+int JackEngine::ClientHasSessionCallback(const char *name)
 {
     JackClientInterface* client = NULL;
     for (int i = 0; i < CLIENT_NUM; i++) {
-        JackClientInterface* client = fClientTable[i];
-        if (client && (strcmp(client->GetClientControl()->fName, name) == 0))
+        client = fClientTable[i];
+        if (client && (strcmp(client->GetClientControl()->fName, name) == 0)) {
             break;
+        }
     }
 
     if (client) {
-        *result = client->GetClientControl()->fCallback[kSessionCallback];
+        return client->GetClientControl()->fCallback[kSessionCallback];
      } else {
-        *result = -1;
+        return -1;
     }
 }
 
