@@ -26,7 +26,7 @@
 
 #include "reserve.h"
 #include "audio_reserve.h"
-#include "JackError.h"
+#include "jack/control.h"
 
 #define DEVICE_MAX 2
 
@@ -47,7 +47,8 @@ SERVER_EXPORT int audio_reservation_init()
     dbus_error_init(&error);
 
     if (!(gConnection = dbus_bus_get(DBUS_BUS_SESSION, &error))) {
-        jack_error("Failed to connect to session bus for device reservation %s\n", error.message);
+        jack_error("Failed to connect to session bus for device reservation: %s\n", error.message);
+        jack_info("To bypass device reservation via session bus, set JACK_NO_AUDIO_RESERVATION=1 prior to starting jackd.\n");
         return -1;
     }
 
@@ -71,11 +72,16 @@ SERVER_EXPORT bool audio_acquire(const char * device_name)
     int ret;
 
     // Open DBus connection first time
-    if (gReserveCount == 0)
-       audio_reservation_init();
+    if (gReserveCount == 0) {
+        if (audio_reservation_init() != 0) {
+            return false;
+        }
+    }
 
     assert(gReserveCount < DEVICE_MAX);
 
+    dbus_error_init(&error);
+
     if ((ret= rd_acquire(
                  &gReservedDevice[gReserveCount].reserved_device,
                  gConnection,
@@ -86,6 +92,7 @@ SERVER_EXPORT bool audio_acquire(const char * device_name)
                  &error)) < 0) {
 
         jack_error("Failed to acquire device name : %s error : %s", device_name, (error.message ? error.message : strerror(-ret)));
+        dbus_error_free(&error);
         return false;
     }
 
@@ -121,8 +128,7 @@ SERVER_EXPORT void audio_release(const char * device_name)
 SERVER_EXPORT void audio_reserve_loop()
 {
     if (gConnection != NULL) {
-       while (dbus_connection_read_write_dispatch (gConnection, -1))
-         ; // empty loop body
+       dbus_connection_read_write_dispatch (gConnection, 200);
     }
 }
 
