@@ -1,6 +1,6 @@
 /* -*- Mode: C ; c-basic-offset: 4 -*- */
 /*
-    Copyright (C) 2007,2008,2010 Nedko Arnaudov
+    Copyright (C) 2007,2008,2010,2011 Nedko Arnaudov
     Copyright (C) 2007-2008 Juuso Alasuutari
 
     This program is free software; you can redistribute it and/or modify
@@ -26,11 +26,14 @@
 #include <string.h>
 #include <dbus/dbus.h>
 #include <assert.h>
+#include <unistd.h>
+#include <errno.h>
 
 #include "controller.h"
 #include "controller_internal.h"
 #include "xml.h"
 #include "reserve.h"
+#include "uptime.h"
 
 struct jack_dbus_interface_descriptor * g_jackcontroller_interfaces[] =
 {
@@ -38,10 +41,12 @@ struct jack_dbus_interface_descriptor * g_jackcontroller_interfaces[] =
     &g_jack_controller_iface_control,
     &g_jack_controller_iface_configure,
     &g_jack_controller_iface_patchbay,
+    &g_jack_controller_iface_session_manager,
     &g_jack_controller_iface_transport,
     NULL
 };
 
+static
 jackctl_driver_t *
 jack_controller_find_driver(
     jackctl_server_t *server,
@@ -64,6 +69,80 @@ jack_controller_find_driver(
     return NULL;
 }
 
+static bool jack_controller_check_slave_driver(struct jack_controller * controller_ptr, const char * name)
+{
+    struct list_head * node_ptr;
+    struct jack_controller_slave_driver * driver_ptr;
+
+    list_for_each(node_ptr, &controller_ptr->slave_drivers)
+    {
+        driver_ptr = list_entry(node_ptr, struct jack_controller_slave_driver, siblings);
+        if (strcmp(name, driver_ptr->name) == 0)
+        {
+            return true;
+        }
+    }
+
+    return false;
+}
+
+static bool jack_controller_load_slave_drivers(struct jack_controller * controller_ptr)
+{
+    struct list_head * node_ptr;
+    struct jack_controller_slave_driver * driver_ptr;
+
+    list_for_each(node_ptr, &controller_ptr->slave_drivers)
+    {
+        driver_ptr = list_entry(node_ptr, struct jack_controller_slave_driver, siblings);
+
+        assert(driver_ptr->handle != NULL);
+        assert(!driver_ptr->loaded);
+
+        if (!jackctl_server_add_slave(controller_ptr->server, driver_ptr->handle))
+        {
+            jack_error("Driver \"%s\" cannot be loaded", driver_ptr->name);
+            return false;
+        }
+
+        driver_ptr->loaded = true;
+    }
+
+    return true;
+}
+
+static void jack_controller_unload_slave_drivers(struct jack_controller * controller_ptr)
+{
+    struct list_head * node_ptr;
+    struct jack_controller_slave_driver * driver_ptr;
+
+    list_for_each(node_ptr, &controller_ptr->slave_drivers)
+    {
+        driver_ptr = list_entry(node_ptr, struct jack_controller_slave_driver, siblings);
+        if (driver_ptr->loaded)
+        {
+            jackctl_server_remove_slave(controller_ptr->server, driver_ptr->handle);
+            driver_ptr->loaded = false;
+        }
+    }
+}
+
+static void jack_controller_remove_slave_drivers(struct jack_controller * controller_ptr)
+{
+    struct jack_controller_slave_driver * driver_ptr;
+
+    while (!list_empty(&controller_ptr->slave_drivers))
+    {
+        driver_ptr = list_entry(controller_ptr->slave_drivers.next, struct jack_controller_slave_driver, siblings);
+        assert(!driver_ptr->loaded);
+        list_del(&driver_ptr->siblings);
+        free(driver_ptr->name);
+        free(driver_ptr);
+    }
+
+    controller_ptr->slave_drivers_vparam_value.str[0] = 0;
+}
+
+static
 jackctl_internal_t *
 jack_controller_find_internal(
     jackctl_server_t *server,
@@ -86,42 +165,18 @@ jack_controller_find_internal(
     return NULL;
 }
 
-jackctl_parameter_t *
-jack_controller_find_parameter(
-    const JSList * parameters_list,
-    const char * parameter_name)
-{
-    while (parameters_list)
-    {
-        if (strcmp(jackctl_parameter_get_name((jackctl_parameter_t *)parameters_list->data), parameter_name) == 0)
-        {
-            return parameters_list->data;
-        }
-
-        parameters_list = jack_slist_next(parameters_list);
-    }
-
-    return NULL;
-}
-
 bool
 jack_controller_select_driver(
     struct jack_controller * controller_ptr,
     const char * driver_name)
 {
-    jackctl_driver_t *driver;
-
-    driver = jack_controller_find_driver(controller_ptr->server, driver_name);
-    if (driver == NULL)
+    if (!jack_params_set_driver(controller_ptr->params, driver_name))
     {
         return false;
     }
 
     jack_info("driver \"%s\" selected", driver_name);
 
-    controller_ptr->driver = driver;
-    controller_ptr->driver_set = true;
-
     return true;
 }
 
@@ -145,22 +200,18 @@ jack_controller_start_server(
 
     assert(!controller_ptr->started); /* should be ensured by caller */
 
-    if (controller_ptr->driver == NULL)
-    {
-        jack_dbus_error(dbus_call_context_ptr, JACK_DBUS_ERROR_GENERIC, "Select driver first!");
-        goto fail;
-    }
-
     controller_ptr->xruns = 0;
 
     if (!jackctl_server_open(
             controller_ptr->server,
-            controller_ptr->driver))
+            jack_params_get_driver(controller_ptr->params)))
     {
         jack_dbus_error(dbus_call_context_ptr, JACK_DBUS_ERROR_GENERIC, "Failed to open server");
         goto fail;
     }
 
+    jack_controller_load_slave_drivers(controller_ptr);
+
     if (!jackctl_server_start(
             controller_ptr->server))
     {
@@ -221,6 +272,8 @@ fail_stop_server:
     }
 
 fail_close_server:
+    jack_controller_unload_slave_drivers(controller_ptr);
+
     if (!jackctl_server_close(controller_ptr->server))
     {
         jack_error("failed to close jack server");
@@ -237,6 +290,15 @@ jack_controller_stop_server(
 {
     int ret;
 
+    pthread_mutex_lock(&controller_ptr->lock);
+    if (!list_empty(&controller_ptr->session_pending_commands))
+    {
+        pthread_mutex_unlock(&controller_ptr->lock);
+        jack_dbus_error(dbus_call_context_ptr, JACK_DBUS_ERROR_GENERIC, "Refusing to stop JACK server because of pending session commands");
+        return false;
+    }
+    pthread_mutex_unlock(&controller_ptr->lock);
+
     jack_info("Stopping jack server...");
 
     assert(controller_ptr->started); /* should be ensured by caller */
@@ -263,6 +325,8 @@ jack_controller_stop_server(
         return FALSE;
     }
 
+    jack_controller_unload_slave_drivers(controller_ptr);
+
     if (!jackctl_server_close(controller_ptr->server))
     {
         jack_dbus_error(dbus_call_context_ptr, JACK_DBUS_ERROR_GENERIC, "Failed to close server");
@@ -279,11 +343,14 @@ jack_controller_switch_master(
     struct jack_controller * controller_ptr,
     void *dbus_call_context_ptr)
 {
+    assert(controller_ptr->started); /* should be ensured by caller */
+
     if (!jackctl_server_switch_master(
             controller_ptr->server,
-            controller_ptr->driver))
+            jack_params_get_driver(controller_ptr->params)))
     {
         jack_dbus_error(dbus_call_context_ptr, JACK_DBUS_ERROR_GENERIC, "Failed to switch master");
+        controller_ptr->started = false;
         return FALSE;
     }
 
@@ -310,6 +377,8 @@ on_device_acquire(const char * device_name)
     int ret;
     DBusError error;
 
+    dbus_error_init(&error);
+
     ret = rd_acquire(
         &g_reserved_device[g_device_count].reserved_device,
         g_connection,
@@ -321,6 +390,7 @@ on_device_acquire(const char * device_name)
     if (ret  < 0)
     {
         jack_error("Failed to acquire device name : %s error : %s", device_name, (error.message ? error.message : strerror(-ret)));
+        dbus_error_free(&error);
         return false;
     }
 
@@ -338,10 +408,10 @@ on_device_release(const char * device_name)
 
     // Look for corresponding reserved device
     for (i = 0; i < DEVICE_MAX; i++) {
- 	if (strcmp(g_reserved_device[i].device_name, device_name) == 0)  
+ 	if (strcmp(g_reserved_device[i].device_name, device_name) == 0)
 	    break;
     }
-   
+
     if (i < DEVICE_MAX) {
 	jack_info("Released audio card %s", device_name);
         rd_release(g_reserved_device[i].reserved_device);
@@ -352,16 +422,109 @@ on_device_release(const char * device_name)
     g_device_count--;
 }
 
+#define controller_ptr ((struct jack_controller *)obj)
+
+static bool slave_drivers_parameter_is_set(void * obj)
+{
+    return controller_ptr->slave_drivers_set;
+}
+
+static bool slave_drivers_parameter_reset(void * obj)
+{
+    if (controller_ptr->started)
+    {
+        jack_error("Cannot modify slave-drivers when server is started");
+        return false;
+    }
+
+    jack_controller_remove_slave_drivers(controller_ptr);
+    controller_ptr->slave_drivers_set = false;
+    return true;
+}
+
+static union jackctl_parameter_value slave_drivers_parameter_get_value(void * obj)
+{
+    return controller_ptr->slave_drivers_vparam_value;
+}
+
+static bool slave_drivers_parameter_set_value(void * obj, const union jackctl_parameter_value * value_ptr)
+{
+    char * buffer;
+    char * save;
+    const char * token;
+    struct list_head old_list;
+    struct list_head new_list;
+    union jackctl_parameter_value old_value;
+    union jackctl_parameter_value new_value;
+    bool old_set;
+
+    if (controller_ptr->started)
+    {
+        jack_error("Cannot modify slave-drivers when server is started");
+        return false;
+    }
+
+    old_set = controller_ptr->slave_drivers_set;
+    old_value = controller_ptr->slave_drivers_vparam_value;
+    controller_ptr->slave_drivers_vparam_value.str[0] = 0;
+    old_list = controller_ptr->slave_drivers;
+    INIT_LIST_HEAD(&controller_ptr->slave_drivers);
+
+    buffer = strdup(value_ptr->str);
+    if (buffer == NULL)
+    {
+        jack_error("strdup() failed.");
+        return false;
+    }
+
+    token = strtok_r(buffer, ",", &save);
+    while (token)
+    {
+        //jack_info("slave driver '%s'", token);
+        if (!jack_controller_add_slave_driver(controller_ptr, token))
+        {
+            jack_controller_remove_slave_drivers(controller_ptr);
+            controller_ptr->slave_drivers = old_list;
+            controller_ptr->slave_drivers_vparam_value = old_value;
+            controller_ptr->slave_drivers_set = old_set;
+
+            free(buffer);
+
+            return false;
+        }
+
+        token = strtok_r(NULL, ",", &save);
+    }
+
+    new_value = controller_ptr->slave_drivers_vparam_value;
+    new_list = controller_ptr->slave_drivers;
+    controller_ptr->slave_drivers = old_list;
+    jack_controller_remove_slave_drivers(controller_ptr);
+    controller_ptr->slave_drivers_vparam_value = new_value;
+    controller_ptr->slave_drivers = new_list;
+    controller_ptr->slave_drivers_set = true;
+
+    free(buffer);
+
+    return true;
+}
+
+static union jackctl_parameter_value slave_drivers_parameter_get_default_value(void * obj)
+{
+    union jackctl_parameter_value value;
+    value.str[0] = 0;
+    return value;
+}
+
+#undef controller_ptr
+
 void *
 jack_controller_create(
         DBusConnection *connection)
 {
+    int error;
     struct jack_controller *controller_ptr;
-    const JSList * node_ptr;
-    const char ** driver_name_target;
-    const char ** internal_name_target;
-    JSList * drivers;
-    JSList * internals;
+    const char * address[PARAM_ADDRESS_SIZE];
     DBusObjectPathVTable vtable =
     {
         jack_dbus_message_handler_unregister,
@@ -376,60 +539,55 @@ jack_controller_create(
         goto fail;
     }
 
-    controller_ptr->server = jackctl_server_create(on_device_acquire, on_device_release);
+    error = pthread_mutex_init(&controller_ptr->lock, NULL);
+    if (error != 0)
+    {
+        jack_error("Failed to initialize mutex. error %d", error);
+        goto fail_free;
+    }
+
+    INIT_LIST_HEAD(&controller_ptr->session_pending_commands);
+
+    controller_ptr->server = jackctl_server_create2(on_device_acquire, on_device_release, NULL);
     if (controller_ptr->server == NULL)
     {
         jack_error("Failed to create server object");
-        goto fail_free;
+        goto fail_uninit_mutex;
+    }
+
+    controller_ptr->params = jack_params_create(controller_ptr->server);
+    if (controller_ptr->params == NULL)
+    {
+        jack_error("Failed to initialize parameter tree");
+        goto fail_destroy_server;
     }
 
     controller_ptr->client = NULL;
     controller_ptr->started = false;
-    controller_ptr->driver = NULL;
-    controller_ptr->driver_set = false;
 
-    drivers = (JSList *)jackctl_server_get_drivers_list(controller_ptr->server);
-    controller_ptr->drivers_count = jack_slist_length(drivers);
-    controller_ptr->driver_names = malloc(controller_ptr->drivers_count * sizeof(const char *));
-    if (controller_ptr->driver_names == NULL)
-    {
-        jack_error("Ran out of memory trying to allocate driver names array");
-        goto fail_destroy_server;
-    }
+    controller_ptr->pending_save = 0;
 
-    driver_name_target = controller_ptr->driver_names;
-    node_ptr = jackctl_server_get_drivers_list(controller_ptr->server);
-    while (node_ptr != NULL)
-    {
-        *driver_name_target = jackctl_driver_get_name((jackctl_driver_t *)node_ptr->data);
+    INIT_LIST_HEAD(&controller_ptr->slave_drivers);
+    controller_ptr->slave_drivers_set = false;
+    controller_ptr->slave_drivers_vparam_value.str[0] = 0;
 
-        /* select default driver */
-        if (controller_ptr->driver == NULL && strcmp(*driver_name_target, DEFAULT_DRIVER) == 0)
-        {
-            controller_ptr->driver = (jackctl_driver_t *)node_ptr->data;
-        }
+    controller_ptr->slave_drivers_vparam.obj = controller_ptr;
 
-        node_ptr = jack_slist_next(node_ptr);
-        driver_name_target++;
-    }
-    
-    internals = (JSList *)jackctl_server_get_internals_list(controller_ptr->server);
-    controller_ptr->internals_count = jack_slist_length(internals);
-    controller_ptr->internal_names = malloc(controller_ptr->internals_count * sizeof(const char *));
-    if (controller_ptr->internal_names == NULL)
-    {
-        jack_error("Ran out of memory trying to allocate internals names array");
-        goto fail_free_driver_names_array;
-    }
+    controller_ptr->slave_drivers_vparam.vtable.is_set = slave_drivers_parameter_is_set;
+    controller_ptr->slave_drivers_vparam.vtable.reset = slave_drivers_parameter_reset;
+    controller_ptr->slave_drivers_vparam.vtable.get_value = slave_drivers_parameter_get_value;
+    controller_ptr->slave_drivers_vparam.vtable.set_value = slave_drivers_parameter_set_value;
+    controller_ptr->slave_drivers_vparam.vtable.get_default_value = slave_drivers_parameter_get_default_value;
 
-    internal_name_target = controller_ptr->internal_names;
-    node_ptr = jackctl_server_get_internals_list(controller_ptr->server);
-    while (node_ptr != NULL)
-    {
-        *internal_name_target = jackctl_internal_get_name((jackctl_internal_t *)node_ptr->data);
-        node_ptr = jack_slist_next(node_ptr);
-        internal_name_target++;
-    }
+    controller_ptr->slave_drivers_vparam.type = JackParamString;
+    controller_ptr->slave_drivers_vparam.name = "slave-drivers";
+    controller_ptr->slave_drivers_vparam.short_decr = "Slave drivers to use";
+    controller_ptr->slave_drivers_vparam.long_descr = "A comma separated list of slave drivers";
+    controller_ptr->slave_drivers_vparam.constraint_flags = 0;
+
+    address[0] = PTNODE_ENGINE;
+    address[1] = NULL;
+    jack_params_add_parameter(controller_ptr->params, address, true, &controller_ptr->slave_drivers_vparam);
 
     controller_ptr->dbus_descriptor.context = controller_ptr;
     controller_ptr->dbus_descriptor.interfaces = g_jackcontroller_interfaces;
@@ -441,22 +599,22 @@ jack_controller_create(
             &controller_ptr->dbus_descriptor))
     {
         jack_error("Ran out of memory trying to register D-Bus object path");
-        goto fail_free_internal_names_array;
+        goto fail_destroy_params;
     }
 
     jack_controller_settings_load(controller_ptr);
 
     return controller_ptr;
 
-fail_free_internal_names_array:
-    free(controller_ptr->internal_names);
-
-fail_free_driver_names_array:
-    free(controller_ptr->driver_names);
+fail_destroy_params:
+    jack_params_destroy(controller_ptr->params);
 
 fail_destroy_server:
     jackctl_server_destroy(controller_ptr->server);
 
+fail_uninit_mutex:
+    pthread_mutex_destroy(&controller_ptr->lock);
+
 fail_free:
     free(controller_ptr);
 
@@ -465,41 +623,106 @@ fail:
 }
 
 bool
-jack_controller_add_slave(
-    struct jack_controller *controller_ptr,
+jack_controller_add_slave_driver(
+    struct jack_controller * controller_ptr,
     const char * driver_name)
 {
-    jackctl_driver_t *driver;
+    jackctl_driver_t * driver;
+    struct jack_controller_slave_driver * driver_ptr;
+    size_t len_old;
+    size_t len_new;
 
-    driver = jack_controller_find_driver(controller_ptr->server, driver_name);
-      
-    if (driver == NULL)
+    len_old = strlen(controller_ptr->slave_drivers_vparam_value.str);
+    len_new = strlen(driver_name);
+    if (len_old + len_new + 2 > sizeof(controller_ptr->slave_drivers_vparam_value.str))
     {
+        jack_error("No more space for slave drivers.");
         return false;
     }
 
-    jack_info("driver \"%s\" selected", driver_name);
+    driver = jack_controller_find_driver(controller_ptr->server, driver_name);
+    if (driver == NULL)
+    {
+        jack_error("Unknown driver \"%s\"", driver_name);
+        return false;
+    }
 
-    return jackctl_server_add_slave(controller_ptr->server, driver);
+    if (jack_controller_check_slave_driver(controller_ptr, driver_name))
+    {
+        jack_info("Driver \"%s\" is already slave", driver_name);
+        return true;
+    }
+
+    driver_ptr = malloc(sizeof(struct jack_controller_slave_driver));
+    if (driver_ptr == NULL)
+    {
+        jack_error("malloc() failed to allocate jack_controller_slave_driver struct");
+        return false;
+    }
+
+    driver_ptr->name = strdup(driver_name);
+    if (driver_ptr->name == NULL)
+    {
+        jack_error("strdup() failed for slave driver name \"%s\"", driver_name);
+        free(driver_ptr);
+        return false;
+    }
+
+    driver_ptr->handle = driver;
+    driver_ptr->loaded = false;
+
+    jack_info("driver \"%s\" set as slave", driver_name);
+
+    list_add_tail(&driver_ptr->siblings, &controller_ptr->slave_drivers);
+
+    if (len_old != 0)
+    {
+        controller_ptr->slave_drivers_vparam_value.str[len_old++] = ',';
+    }
+
+    memcpy(controller_ptr->slave_drivers_vparam_value.str + len_old, driver_name, len_new + 1);
+    controller_ptr->slave_drivers_set = true;
+
+    return true;
 }
 
 bool
-jack_controller_remove_slave(
-    struct jack_controller *controller_ptr,
+jack_controller_remove_slave_driver(
+    struct jack_controller * controller_ptr,
     const char * driver_name)
 {
-    jackctl_driver_t *driver;
+    struct list_head * node_ptr;
+    struct jack_controller_slave_driver * driver_ptr;
 
-    driver = jack_controller_find_driver(controller_ptr->server, driver_name);
-      
-    if (driver == NULL)
+    list_for_each(node_ptr, &controller_ptr->slave_drivers)
     {
-        return false;
+        driver_ptr = list_entry(node_ptr, struct jack_controller_slave_driver, siblings);
+        if (strcmp(driver_ptr->name, driver_name) == 0)
+        {
+            list_del(&driver_ptr->siblings);
+            free(driver_ptr->name);
+            free(driver_ptr);
+
+            /* update the slave-drivers param value */
+            controller_ptr->slave_drivers_vparam_value.str[0] = 0;
+            list_for_each(node_ptr, &controller_ptr->slave_drivers)
+            {
+                driver_ptr = list_entry(node_ptr, struct jack_controller_slave_driver, siblings);
+                if (controller_ptr->slave_drivers_vparam_value.str[0] != 0)
+                {
+                    strcat(controller_ptr->slave_drivers_vparam_value.str, ",");
+                }
+
+                strcat(controller_ptr->slave_drivers_vparam_value.str, driver_ptr->name);
+            }
+
+            jack_info("driver \"%s\" is not slave anymore", driver_name);
+
+            return true;
+        }
     }
 
-    jack_info("driver \"%s\" selected", driver_name);
-
-    return jackctl_server_remove_slave(controller_ptr->server, driver);
+    return false;
 }
 
 bool
@@ -546,14 +769,59 @@ jack_controller_destroy(
 {
     if (controller_ptr->started)
     {
-        jack_controller_stop_server(controller_ptr, NULL);
+        while (!jack_controller_stop_server(controller_ptr, NULL))
+        {
+            jack_info("jack server failed to stop, retrying in 3 seconds...");
+            usleep(3000000);
+        }
     }
 
-    free(controller_ptr->driver_names);
-    free(controller_ptr->internal_names);
-
+    jack_controller_remove_slave_drivers(controller_ptr);
+    jack_params_destroy(controller_ptr->params);
     jackctl_server_destroy(controller_ptr->server);
-
+    pthread_mutex_destroy(&controller_ptr->lock);
     free(controller_ptr);
 }
 
+void
+jack_controller_run(
+    void * context)
+{
+    long ut;
+
+    if (controller_ptr->pending_save == 0)
+    {
+        return;
+    }
+
+    if ((ut = uptime()) < 0)
+    {
+        jack_error(UPTIME_FUNCTION_NAME "() failed with %d", errno);
+    }
+    else if (ut < controller_ptr->pending_save + 2) /* delay save by two seconds */
+    {
+        return;
+    }
+
+    controller_ptr->pending_save = 0;
+    jack_controller_settings_save_auto(controller_ptr);
+}
+
+#undef controller_ptr
+
+void
+jack_controller_pending_save(
+    struct jack_controller * controller_ptr)
+{
+    long ut;
+
+    if ((ut = uptime()) < 0)
+    {
+        jack_error(UPTIME_FUNCTION_NAME "() failed with %d.", errno);
+        controller_ptr->pending_save = 0;
+        jack_controller_settings_save_auto(controller_ptr);
+        return;
+    }
+
+    controller_ptr->pending_save = ut;
+}
