@@ -1,5 +1,6 @@
 /*
 Copyright (C) 2004-2008 Grame
+Copyright (C) 2016-2023 Filipe Coelho <falktx@falktx.com>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
@@ -90,11 +91,19 @@ bool JackMachSemaphore::Wait()
         return false;
     }
 
-    kern_return_t res;
-    if ((res = semaphore_wait(fSemaphore)) != KERN_SUCCESS) {
-        jack_error("JackMachSemaphore::Wait name = %s err = %s", fName, mach_error_string(res));
+    kern_return_t res = semaphore_wait(fSemaphore);
+
+    if (res == KERN_SUCCESS) {
+        return true;
     }
-    return (res == KERN_SUCCESS);
+
+    // killing a thread will abort the semaphore wait, skip the error log
+    if (res == KERN_ABORTED) {
+        return false;
+    }
+
+    jack_error("JackMachSemaphore::Wait name = %s err = %s", fName, mach_error_string(res));
+    return false;
 }
 
 bool JackMachSemaphore::TimedWait(long usec)
@@ -104,15 +113,23 @@ bool JackMachSemaphore::TimedWait(long usec)
         return false;
     }
 
-    kern_return_t res;
     mach_timespec time;
     time.tv_sec = usec / 1000000;
     time.tv_nsec = (usec % 1000000) * 1000;
 
-    if ((res = semaphore_timedwait(fSemaphore, time)) != KERN_SUCCESS) {
-        jack_error("JackMachSemaphore::TimedWait name = %s usec = %ld err = %s", fName, usec, mach_error_string(res));
+    kern_return_t res = semaphore_timedwait(fSemaphore, time);
+
+    if (res == KERN_SUCCESS) {
+        return true;
     }
-    return (res == KERN_SUCCESS);
+
+    // killing a thread will abort the semaphore wait, skip the error log
+    if (res == KERN_ABORTED) {
+        return false;
+    }
+
+    jack_error("JackMachSemaphore::TimedWait name = %s usec = %ld err = %s", fName, usec, mach_error_string(res));
+    return false;
 }
 
 /*! \brief Server side: create semaphore and publish IPC primitives to make it accessible.
@@ -305,7 +322,7 @@ bool JackMachSemaphore::ConnectInput(const char* client_name, const char* server
     } else {
         fSemaphore = msg.hdr.msgh_remote_port;
 
-        jack_log("JackMachSemaphore::Connect: OK, name = %s ", fName);
+        jack_log("JackMachSemaphore::Connect: OK, name = %s", fName);
         return true;
     }
 }
@@ -353,17 +370,22 @@ bool JackMachSemaphore::Disconnect()
 // Server side : destroy the JackGlobals
 void JackMachSemaphore::Destroy()
 {
+    const mach_port_t task = mach_task_self();
     kern_return_t res;
-    mach_port_t task = mach_task_self();
 
     if (fSemaphore == MACH_PORT_NULL) {
         jack_error("JackMachSemaphore::Destroy semaphore is MACH_PORT_NULL; already destroyed?");
         return;
     }
 
+    if (fSemServer && fSemServer->Invalidate()) {
+        fServicePort = MACH_PORT_NULL;
+        fSemaphore = MACH_PORT_NULL;
+    }
+
     if (fThreadSemServer) {
-        if (fThreadSemServer->Kill() < 0) {
-            jack_error("JackMachSemaphore::Destroy failed to kill semaphore server thread...");
+        if (fThreadSemServer->Stop() < 0) {
+            jack_error("JackMachSemaphore::Destroy failed to stop semaphore server thread...");
             // Oh dear. How sad. Never mind.
         }
 
@@ -378,16 +400,20 @@ void JackMachSemaphore::Destroy()
         delete server;
     }
 
-    if ((res = mach_port_destroy(task, fServicePort)) != KERN_SUCCESS) {
-        jack_mach_error(res, "failed to destroy IPC port");
-    } else {
-        fServicePort = MACH_PORT_NULL;
+    if (fServicePort != MACH_PORT_NULL) {
+        if ((res = mach_port_destroy(task, fServicePort)) != KERN_SUCCESS) {
+            jack_mach_error(res, "failed to destroy IPC port");
+        } else {
+            fServicePort = MACH_PORT_NULL;
+        }
     }
 
-    if ((res = semaphore_destroy(mach_task_self(), fSemaphore)) != KERN_SUCCESS) {
-        jack_mach_error(res, "failed to destroy semaphore");
-    } else {
-        fSemaphore = MACH_PORT_NULL;
+    if (fSemaphore != MACH_PORT_NULL) {
+        if ((res = semaphore_destroy(mach_task_self(), fSemaphore)) != KERN_SUCCESS) {
+            jack_mach_error(res, "failed to destroy semaphore");
+        } else {
+            fSemaphore = MACH_PORT_NULL;
+        }
     }
 
     jack_log("JackMachSemaphore::Destroy: OK, name = %s", fName);
