@@ -13,7 +13,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with this program; if not, write to the Free Software 
+along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 */
@@ -26,7 +26,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 static double __jack_time_ratio;
 
-SERVER_EXPORT void JackSleep(long usec) 
+SERVER_EXPORT void JackSleep(long usec)
 {
 	usleep(usec);
 }
@@ -36,18 +36,21 @@ SERVER_EXPORT void InitTime()
 {
 	mach_timebase_info_data_t info;
 	mach_timebase_info(&info);
-	__jack_time_ratio = ((float)info.numer / info.denom) / 1000;
+	__jack_time_ratio = ((double)info.numer / (double)info.denom) / 1000;
 }
 
-SERVER_EXPORT jack_time_t GetMicroSeconds(void) 
-{
-    return (jack_time_t) (mach_absolute_time () * __jack_time_ratio);
-}
-
-SERVER_EXPORT void SetClockSource(jack_timer_type_t source)
+SERVER_EXPORT void EndTime()
 {}
 
-SERVER_EXPORT const char* ClockSourceName(jack_timer_type_t source)
+SERVER_EXPORT jack_time_t GetMicroSeconds(void)
+{
+    return (jack_time_t) (mach_absolute_time() * __jack_time_ratio);
+}
+
+void SetClockSource(jack_timer_type_t source)
+{}
+
+const char* ClockSourceName(jack_timer_type_t source)
 {
     return "";
 }
