@@ -1,20 +1,20 @@
 /*
  Copyright (C) 2004-2008 Grame
- 
+
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 2.1 of the License, or
  (at your option) any later version.
- 
+
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.
- 
+
  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- 
+
  */
 
 
@@ -23,7 +23,17 @@
 
 #define jack_server_dir "server"
 #define jack_client_dir "client"
-#define ADDON_DIR "jack"
+#define JACK_DEFAULT_DRIVER "portaudio"
+#define JACK_LOCATION "C:\\Program Files\\JACK2"
+
+#ifndef ADDON_DIR
+    #define ADDON_DIR "jack"
+#endif
+
+#define MAKE_WIDECHAR_CONSTANT2(Quote) L##Quote
+#define MAKE_WIDECHAR_CONSTANT(Quote) MAKE_WIDECHAR_CONSTANT2(Quote)
+#define ADDON_DIRW MAKE_WIDECHAR_CONSTANT(ADDON_DIR)
+
 
 namespace Jack
 {
@@ -62,7 +72,7 @@ namespace Jack { typedef JackWinNamedPipe JackChannelTransaction; }
 #include "JackWinProcessSync.h"
 namespace Jack { typedef JackWinProcessSync JackProcessSync; }
 
-/* __JackPlatformServerChannel__ */ 
+/* __JackPlatformServerChannel__ */
 #include "JackWinNamedPipeServerChannel.h"
 namespace Jack { typedef JackWinNamedPipeServerChannel JackServerChannel; }
 
