@@ -1,32 +1,37 @@
 /*
  Copyright (C) 2004-2008 Grame
- 
+
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 2.1 of the License, or
  (at your option) any later version.
- 
+
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.
- 
+
  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- 
+
  */
 
 
 #ifndef __JackWinNamedPipe__
 #define __JackWinNamedPipe__
 
+#ifdef __MINGW32__
+#include <winsock2.h>
+#endif
 #include <windows.h>
 
+#include "JackChannel.h"
+
 namespace Jack
 {
 
-class JackWinNamedPipe
+class JackWinNamedPipeAux
 {
 
     protected:
@@ -34,32 +39,62 @@ class JackWinNamedPipe
         HANDLE fNamedPipe;
         char fName[256];
 
+        int ReadAux(void* data, int len);
+        int WriteAux(void* data, int len);
+
     public:
 
-        JackWinNamedPipe(): fNamedPipe(INVALID_HANDLE_VALUE)
+        JackWinNamedPipeAux(): fNamedPipe(INVALID_HANDLE_VALUE)
         {}
-        JackWinNamedPipe(HANDLE pipe): fNamedPipe(pipe)
+        JackWinNamedPipeAux(HANDLE pipe): fNamedPipe(pipe)
+        {}
+        virtual ~JackWinNamedPipeAux()
+        {}
+
+};
+
+
+class JackWinNamedPipe : public JackWinNamedPipeAux, public detail::JackChannelTransactionInterface
+{
+
+    public:
+
+        JackWinNamedPipe():JackWinNamedPipeAux()
+        {}
+        JackWinNamedPipe(HANDLE pipe):JackWinNamedPipeAux(pipe)
         {}
         virtual ~JackWinNamedPipe()
         {}
 
-        virtual int Read(void* data, int len);
-        virtual int Write(void* data, int len);
+        virtual int Read(void* data, int len)
+        {
+            return ReadAux(data, len);
+        }
+        virtual int Write(void* data, int len)
+        {
+            return WriteAux(data, len);
+        }
 };
 
 /*!
 \brief Client named pipe.
 */
 
-class JackWinNamedPipeClient : public JackWinNamedPipe
+class JackWinNamedPipeClient : public JackWinNamedPipeAux, public detail::JackClientRequestInterface
 {
 
+    protected:
+
+        int ConnectAux();
+
     public:
 
-        JackWinNamedPipeClient(): JackWinNamedPipe()
-        {}
-        JackWinNamedPipeClient(HANDLE pipe): JackWinNamedPipe(pipe)
+        JackWinNamedPipeClient():JackWinNamedPipeAux()
         {}
+        JackWinNamedPipeClient(HANDLE pipe, const char* name):JackWinNamedPipeAux(pipe)
+        {
+            strcpy(fName, name);
+        }
 
         virtual ~JackWinNamedPipeClient()
         {}
@@ -67,8 +102,20 @@ class JackWinNamedPipeClient : public JackWinNamedPipe
         virtual int Connect(const char* dir, int which);
         virtual int Connect(const char* dir, const char* name, int which);
         virtual int Close();
+
+        virtual int Read(void* data, int len)
+        {
+            return ReadAux(data, len);
+        }
+        virtual int Write(void* data, int len)
+        {
+            return WriteAux(data, len);
+        }
+
         virtual void SetReadTimeOut(long sec);
         virtual void SetWriteTimeOut(long sec);
+        
+        virtual void SetNonBlocking(bool onoff);
 };
 
 class JackWinAsyncNamedPipeClient : public JackWinNamedPipeClient
@@ -84,7 +131,7 @@ class JackWinAsyncNamedPipeClient : public JackWinNamedPipeClient
     public:
 
         JackWinAsyncNamedPipeClient();
-        JackWinAsyncNamedPipeClient(HANDLE pipe, bool pending);
+        JackWinAsyncNamedPipeClient(HANDLE pipe, const char* name, bool pending);
         virtual ~JackWinAsyncNamedPipeClient();
 
         virtual int Read(void* data, int len);
@@ -114,6 +161,9 @@ class JackWinAsyncNamedPipeClient : public JackWinNamedPipeClient
 
 class JackWinNamedPipeServer : public JackWinNamedPipe
 {
+    private:
+
+        int BindAux();
 
     public:
 
@@ -136,6 +186,10 @@ class JackWinNamedPipeServer : public JackWinNamedPipe
 class JackWinAsyncNamedPipeServer : public JackWinNamedPipeServer
 {
 
+    private:
+
+        int BindAux();
+
     public:
 
         JackWinAsyncNamedPipeServer(): JackWinNamedPipeServer()
